use strict;
use locale;

if ($#ARGV != 0){
    die "Usage : ", $0, " <chemin de lefff-2.1.txt>";
}
open(LEFFF, "<", $ARGV[0]) or die "impossible d'ouvrir le fichier ", $ARGV[0];

my %mode = ("P" => "i",
	    "F" => "i",
	    "I" => "i",
	    "J" => "i",
	    "C" => "c",
	    "Y" => "m",
	    "S" => "s",
	    "T" => "s",
	    "K" => "p",
	     "G" => "p",
	    "W" => "n",
	   );
my %temps = ("P" => "p",
	     "F" => "f",
	     "I" => "i",
	     "J" => "s",
	     "C" => "p",
	     "Y" => "p",
	     "S" => "p",
	     "T" => "i",
	     "K" => "s",
	     "G" => "p",
	     "W" => "-",
	    );

while (my $ligne = <LEFFF>){
    chomp($ligne);
    if ($ligne !~ /^\p{L}/){
	# ces entres ne sont pas des mots
	# warn $ligne, " n'est pas un mot";
	next;
    }
    my ($forme, $count, $cat, $traits) = split(/\t/, $ligne);
    if ( $forme =~ /_$/){
	# ces entres sont des affixes
	# warn $ligne, " affixe";
	next;
    }
    my $lemme;
    if($traits =~ /pred=\'(.+?)___/){
	$lemme = $1;
    }
    elsif($cat eq "auxAvoir"){
	$lemme = "avoir";
    }
    elsif ($cat eq "auxEtre"){
	$lemme = "tre";
    }
    else {
	# warn $ligne, " n'a pas de lemme";
	next;
    }
    my $morphosynt;
    if($traits =~ /@([^@]*)\]/){
	$morphosynt = $1;
    }
    if ($cat eq "adv"){
	print join("\t", $forme, $lemme, "Rgp"), "\n";
    }
    elsif ($cat eq "advneg"){
	print join("\t", $forme, $lemme, "Rgn"), "\n";
    }
    elsif ($cat eq "prep"){
	$forme =~ s/__prep$//;
	print join("\t", $forme, $lemme, "Sp"), "\n";
    }
    elsif ($cat eq "coo"){
	print join("\t", $forme, $lemme, "Cs"), "\n";
    }
    elsif($cat eq "nc" || $cat eq "adj"){
	my @genre = ("f", "m");
	my @nb = ("p", "s");
	if (defined($morphosynt)){
	    if ($morphosynt =~ /s/){
		@nb = ("s");	
	    }
	    if ($morphosynt =~ /p/){
		@nb = ("p");	
	    }
	    if ($morphosynt =~ /f/){
		@genre = ("f");	
	    }
	    if ($morphosynt =~ /m/){
		@genre = ("m");	
	    }
	}
	my $tag;
	if($cat eq "nc"){
	    $tag = "Nc";
	}
	if($cat eq "adj"){
	    $tag = "Afp";
	}
	foreach my $g (@genre){
	    foreach my $n (@nb){
		print join("\t", $forme, $lemme, join("", $tag, $g, $n)), "\n";
	    }
	}
    }
    elsif ($cat eq "v" || $cat eq "auxAvoir" || $cat eq "auxEtre"){
	if ($morphosynt =~ /^([PFIJCYSTKGW]+)([123]*)([mf]*)([ps]*)$/){
	    my $mode_temps = $1;
	    my $personnes = $2;
	    my $genres = $3;
	    my $nb = $4;
		    
	    my @xmode_temps = split(//, $mode_temps);
	    my @xpersonnes = split(//, $personnes);
	    my @xgenres = split(//, $genres);
	    my @xnb = split(//, $nb);
	    
	    my @lmode = ("-");
	    my @ltemps = ("-");
	    my @lpersonnes = ("-");
	    my @lgenres ;
	    if ($mode_temps =~ /K/){
		@lgenres = ("f", "m");
	    }
	    else {
		@lgenres = ("-");
	    }
	    my @lnb ;
	    if ($mode_temps =~ /K/){
		@lnb = ("p", "s");
	    }
	    else {
		@lnb = ("-");
	    }
	    if (@xmode_temps){
		@lmode = map {$mode{ $_ };} @xmode_temps;
		@ltemps = map {$temps{$_};} @xmode_temps;
	    }
	    if (@xpersonnes){
		@lpersonnes = @xpersonnes;
	    }
	    if (@xgenres){
		@lgenres = @xgenres;
	    }
	    if (@xnb){
		@lnb= @xnb;
	    }
	    $lemme =~ s/:er/er/;	    
	    foreach my $m (@lmode){
		foreach my $t (@ltemps){
		    foreach my $p (@lpersonnes){
			foreach my $g (@lgenres){
			    foreach my $n (@lnb){
				if ($cat eq "v"){
				    print join("\t", $forme, $lemme, join("", "Vm", $m, $t, $p, $n, $g)), "\n";
				}
				elsif ($cat eq "auxAvoir"){
				    print join("\t", $forme, $lemme, join("", "Va", $m, $t, $p, $n, $g)), "\n";
				}
				elsif ($cat eq "auxETre"){
				    print join("\t", $forme, $lemme, join("", "Va", $m, $t, $p, $n, $g)), "\n";
				}
			    }
			}
		    }
		}
	    }
	}
	else {
	    # l'tiquette n'a pas le format attendu
	    # warn $ligne , " problme de format pour les traits du verbe";   
	}
    }
    elsif ($cat eq "pres"){
	print join("\t", $forme, $lemme, "F"), "\n";
    }
    elsif ($cat eq "ncpred" || $cat eq "np"){
	# les noms propres sont limins
	# warn $ligne, " nom propre ou nom prdicatif";
    }
    else {
	# les autres catgories ne sont pas traites
	# warn $ligne, " catgorie non traite";
    }
}

