use strict;
use locale;
use XML::Parser;

if ($#ARGV != 0){
    die "Usage : ", $0, " <chemin de Morphalou_1.0.1.xml>\n";
}

open (RES, ">",  "morphalou.txt") or 
    die "Impossible de crer morphalou.txt : ", $!, "\n";

my $parseur = new XML::Parser;
$parseur -> setHandlers(Start => \&balise_ouvrante);
$parseur -> parsefile($ARGV[0]) or 
    die "Problme d'ouverture de ", $ARGV[0], " : ", $!, "\n";

my $categorie;
my $lemme;
my %attributs_lemme;
sub balise_ouvrante{
    my $p = shift(@_);
    my $balise = shift(@_);
    my %attributs = @_;
    if ($balise eq "lexicalEntry"){	
	$categorie = $attributs{"grammaticalCategory"};
	$lemme = $attributs{"lemma"};
	%attributs_lemme = %attributs;
    }
    elsif ($balise eq "inflection"){
	my $forme = $attributs{"orthography"};
	my $tag;
	if ($categorie eq "adverb"){
	    $tag = "Rgp";
	    print RES $forme, "\t", $lemme, "\t", $tag, "\n";
	}
	elsif ($categorie eq "interjection"){
	    $tag = "I";
	    print RES $forme, "\t", $lemme, "\t", $tag, "\n";
	}
	elsif ($categorie eq "onomatopoeia"){
	    $tag = "X";
	    print RES $forme, "\t", $lemme, "\t", $tag, "\n";
	}
	elsif ($categorie eq "functionWord"){
	    $tag = "F";
	    print RES $forme, "\t", $lemme, "\t", $tag, "\n";
	}
	elsif ($categorie eq "commonNoun"){
	    my $genre = $attributs_lemme{"grammaticalGender"};
	    if (not defined($genre)){
		$genre = $attributs{"grammaticalGender"};
	    }
	    $tag = "Nc";
	    if ($genre eq "masculine"){
		$tag .= "m";
	    }
	    elsif ($genre eq "feminine"){ 
		$tag .= "f";
	    }
	    else {
		$tag .="-";
	    }
	    if ($attributs{"grammaticalNumber"} eq "singular"){
		$tag .= "s";
	    }
	    elsif ($attributs{"grammaticalNumber"} eq "plural"){
		$tag .= "p";
	    }
	    else {
		$tag .= "-";
	    }
	    print RES $forme, "\t", $lemme, "\t", $tag, "\n";
	}
	elsif ($categorie eq "adjective"){
	    $tag = "Afp";
	    my $genre = $attributs{"grammaticalGender"};
	    if (not defined($genre)){
		$genre = $attributs_lemme{"grammaticalGender"};
	    }
	    if ($genre eq "masculine"){
		$tag .= "m";
	    }
	    elsif ($genre eq "feminine"){ 
		$tag .= "f";
	    }
	    else {
		$tag .= "-";
	    }
	    if ($attributs{"grammaticalNumber"} eq "singular"){
		$tag .= "s";
	    }
	    elsif ($attributs{"grammaticalNumber"} eq "plural"){
		$tag .= "p";
	    }
	    else {
		$tag .= "-";
	    }
	    print RES $forme, "\t", $lemme, "\t", $tag, "\n";
	}
	elsif ($categorie eq "verb"){
	    my $mode = $attributs{"grammaticalMood"};
	    my $temps = $attributs{"grammaticalTense"};
	    my $personne = $attributs{"grammaticalPerson"};
	    my $nombre = $attributs{"grammaticalNumber"};
	    my $genre = $attributs{"grammaticalGender"};
	    $tag = "Vm";
	    if ($mode eq "indicative"){
		$tag .= "i";
	    }
	    elsif ($mode eq "subjunctive"){
		$tag .= "s";
	    }
	    elsif ($mode eq "imperative"){
		$tag .= "m";
	    }
	    elsif ($mode eq "conditional"){
		$tag .= "c";
	    }
	    elsif ($mode eq "infinitive"){
		$tag .= "n";
	    }
	    elsif ($mode eq "participle"){
		$tag .= "p";
	    }
	    else {
		$tag .= "-";
	    }
	    if ($temps eq "present"){
		$tag .= "p";
	    } 
	    elsif ($temps eq "past"){
		$tag .= "s";
	    }
	    elsif ($temps eq "imperfect"){
		$tag .= "i";
	    }
	    elsif ($temps eq "simplePast"){
		$tag .= "s";
	    }
	    elsif ($temps eq "future"){
		$tag .= "f";
	    }
	    else {
		$tag .="-";
	    }
	    if ($personne eq "firstPerson"){
		$tag .= "1";
 		} 
	    elsif ($personne eq "secondPerson"){
		$tag .= "2";
 		} 
	    elsif ($personne eq "thirdPerson"){
		$tag .= "3";
 		} 
	    else {
		$tag .= "-";
	    }
	    if ($nombre eq "singular"){
		$tag .= "s";
 		} 
	    elsif ($nombre eq "plural"){
		$tag .= "p";
	    } 
	    else {
		$tag .= "-";
	    }
	    if ($genre eq "masculine"){
 		    $tag .= "m";
 		} 
	    elsif ($genre eq "feminine"){
 		    $tag .= "f";
 		} 
	    else {
		$tag .= "-";
	    }
	    print RES $forme, "\t", $lemme, "\t", $tag, "\n";
	}
    }	
}
